package org.unitedfront2.domain.communication;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.unitedfront2.dao.CommunityDao;
import org.unitedfront2.domain.DomainFactory;
import org.unitedfront2.domain.User;
import org.unitedfront2.domain.accesscontrol.AbstractResource;
import org.unitedfront2.domain.accesscontrol.AccessControl;
import org.unitedfront2.domain.accesscontrol.AccessDeniedException;
import org.unitedfront2.domain.accesscontrol.OwnerOnly;

/**
 * ۓI {@link Community} łB
 *
 * @author kurokkie
 *
 */
public abstract class AbstractCommunity extends AbstractResource
    implements Community, Serializable {

    /** VAԍ */
    private static final long serialVersionUID = -5643140420657410489L;

    /** hCt@Ng */
    private transient DomainFactory domainFactory;

    /**
     * ID
     *
     * @invariant ӂȒl
     */
    private Integer id;

    /**
     * R[h
     *
     * @invariant ӂȒl
     */
    private String code;

    /** Tv */
    private Message overview;

    /** eɑ΂ANZX */
    private AccessControl postAccessControl;

    /** ŏIXV */
    private transient Date lastUpdateDate;

    /** XbhXg */
    private transient List<Thread> threads;

    /** R~jeBf[^ANZXIuWFNg */
    private transient CommunityDao communityDao;

    public AbstractCommunity() {
        super();
    }

    public AbstractCommunity(String code, Message overview, Integer ownerId,
            AccessControl readAccessControl, AccessControl writeAccessControl,
            AccessControl postAccessControl) {
        super(ownerId, readAccessControl, writeAccessControl);
        this.code = code;
        this.overview = overview;
        this.postAccessControl = postAccessControl;
    }

    public AbstractCommunity(Integer id, String code, Message overview,
            Integer ownerId, AccessControl readAccessControl,
            AccessControl writeAccessControl, AccessControl postAccessControl) {
        this(code, overview, ownerId, readAccessControl, writeAccessControl,
                postAccessControl);
        this.id = id;
    }

    @Override
    protected final boolean buildEqualsBuilder(EqualsBuilder eb, Object other) {
        if (!(other instanceof Community)) {
            return false;
        }
        Community castOther = (Community) other;
        eb.append(id, castOther.getId())
            .append(code, castOther.getCode())
            .append(overview, castOther.getOverview());
        if (!super.buildEqualsBuilder(eb, other)) {
            return false;
        }
        eb.append(postAccessControl, castOther.getPostAccessControl());
        return super.buildEqualsBuilder(eb, other);
    }

    @Override
    protected final void buildHashCodeBuilder(HashCodeBuilder hcb) {
        hcb.append(id).append(code).append(getClass()).append(overview);
        super.buildHashCodeBuilder(hcb);
        hcb.append(postAccessControl);
    }

    @Override
    protected final void buildToStringBuilder(ToStringBuilder tsb) {
        tsb.append("id", id).append("code", code).append("overview", overview);
        super.buildToStringBuilder(tsb);
        tsb.append("postAccessControl", postAccessControl);
    }

    @Override
    public boolean identify(Community community) {
        if (id == null) {
            return false;
        }
        return id.equals(community.getId());
    }

    @Override
    public void store() throws CommunityCodeUsedByOtherException {
        Community c = communityDao.findByCode(code);
        if (c != null && !c.identify(this)) {
            String message = "The code '" + code
                + "' has already been used by other.";
            logger.warn(message);
            throw new CommunityCodeUsedByOtherException(message);
        }
        setWriteAccessControl(domainFactory.prototype(OwnerOnly.class));
        getOverview().setOwnerId(getOwnerId());
        getOverview().setAuthorId(getOwnerId());
        if (id == null) {
            communityDao.register(this);
            communityDao.registerMyCommunity(getOwnerId(), this.getId());
        } else {
            communityDao.update(this);

            // Qƌ[U̓R~jeBމ
            for (Integer userId : communityDao.findCommunityUserIds(this.id)) {
                if (!canRead(userId)) {
                    try {
                        cancel(userId);
                    } catch (CannotCancelCommunityException e) {
                        // N肦Ȃ
                        logger.error(e.getMessage(), e);
                        throw new IllegalStateException(e);
                    }
                }
            }
        }
    }

    @Override
    public void delete() throws CannotDeleteCommunityException {
        if (!isDeletable()) {
            String message = "The community '" + this + "' cannot delete.";
            logger.warn(message);
            throw new CannotDeleteCommunityException(message);
        }
        communityDao.delete(id);
    }

    @Override
    public boolean isMyCommunity(User user) {
        if (user == null) {
            return false;
        } else {
            List<Integer> users = communityDao.findCommunityUserIds(this.id);
            return users.contains(user.getId());
        }
    }

    @Override
    public void postAccess() throws AccessDeniedException {
        postAccessControl.access(this);
    }

    @Override
    public void postAccess(int userId) throws AccessDeniedException {
        postAccessControl.access(this, userId);
    }

    @Override
    public void postAccess(User user) throws AccessDeniedException {
        if (user == null) {
            postAccess();
        } else {
            postAccess(user.getId());
        }
    }

    @Override
    public boolean canPost() {
        try {
            postAccess();
            return true;
        } catch (AccessDeniedException e) {
            return false;
        }
    }

    @Override
    public boolean canPost(int userId) {
        try {
            postAccess(userId);
            return true;
        } catch (AccessDeniedException e) {
            return false;
        }
    }

    @Override
    public boolean canPost(User user) {
        if (user == null) {
            return canPost();
        } else {
            return canPost(user.getId());
        }
    }

    @Override
    public void entry(int userId) throws CannotEntryCommunityException {
        if (communityDao.findMyCommunies(userId).contains(this)) {
            String message = "The user " + userId
                + " has already been a member of this community '" + this.code
                + "'.";
            logger.warn(message);
            throw new CannotEntryCommunityException(message);
        }
        communityDao.registerMyCommunity(userId, this.id);
    }

    @Override
    public void entry(User user) throws CannotEntryCommunityException {
        if (user == null) {
            String message
                = "The anonymous user cannot entry into the community.";
            logger.warn(message);
            throw new CannotEntryCommunityException(message);
        }
        entry(user.getId());
    }

    @Override
    public void cancel(int userId) throws CannotCancelCommunityException {
        if (getOwnerId().intValue() == userId) {
            String message = "The user " + userId
                + " is the owner of this community '" + this.code + "'.";
            logger.warn(message);
            throw new CannotCancelCommunityException(message);
        }
        if (!communityDao.findCommunityUserIds(this.id).contains(userId)) {
            String message = "The user " + userId
                + " is not member of this community '" + this.code + "'";
            logger.warn(message);
            throw new CannotCancelCommunityException(message);
        }
        communityDao.deleteMyCommunity(userId, this.id);
    }

    @Override
    public void cancel(User user) throws CannotCancelCommunityException {
        if (user == null) {
            String message = "The anonymous user have no community.";
            logger.warn(message);
            throw new CannotCancelCommunityException(message);
        }
        cancel(user.getId());
    }

    @Override
    public Date getLastUpdateDate() {
        if (this.lastUpdateDate == null) {
            return null;
        } else {
            return (Date) this.lastUpdateDate.clone();
        }
    }

    @Override
    public void setLastUpdateDate(Date lastUpdateDate) {
        if (lastUpdateDate == null) {
            this.lastUpdateDate = null;
        } else {
            this.lastUpdateDate = (Date) lastUpdateDate.clone();
        }
    }

    @Override
    public final <C extends Community> C changeTo(Class<C> clazz)
        throws CannotChangeCommunityException {
        if (!canChangeTo(clazz)) {
            String message = "Cannot change this community '" + this.code
                + "' to clazz '" + clazz.getClass().getName() + "'.";
            logger.warn(message);
            throw new CannotChangeCommunityException(message);
        }
        return doChangeTo(clazz);
    }

    @Override
    public <C extends Community> C changeTo(String className)
        throws CannotChangeCommunityException {
        Class<C> clazz;
        try {
            clazz = (Class<C>) Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new IllegalArgumentException(e);
        }
        return changeTo(clazz);
    }

    @Override
    public <C extends Community> boolean canChangeTo(String className) {
        Class<C> clazz;
        try {
            clazz = (Class<C>) Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new IllegalArgumentException(e);
        }
        return canChangeTo(clazz);
    }

    @Override
    public Thread getThread() {
        List<Thread> threads = communityDao.findThreads(id);
        if (threads.isEmpty()) {
            return null;
        } else {
            return threads.get(0);
        }
    }

    @Override
    public Thread getThread(Integer id) {
        if (id == null) {
            return getThread();
        } else {
            return findThread(id);
        }
    }

    @Override
    public Thread findThread(int threadId) {
        return communityDao.findThread(this.id, threadId);
    }

    @Override
    public Thread findThreadForPost(int id, User user)
        throws AccessDeniedException {
        Thread t = findThread(id);
        if (t == null) {
            return null;
        } else {
            t.readAccess(user);
            return t;
        }
    }

    @Override
    public Thread findThreadForRead(int id, User user)
        throws AccessDeniedException {
        Thread t = findThread(id);
        if (t == null) {
            return null;
        } else {
            t.writeAccess(user);
            return t;
        }
    }

    @Override
    public Thread findThreadForWrite(int id, User user)
        throws AccessDeniedException {
        Thread t = findThread(id);
        if (t == null) {
            return null;
        } else {
            t.postAccess(user);
            return t;
        }
    }

    @Override
    public List<Thread> getThreads() {
        if (this.threads == null && getId() != null
                && getCommunityDao() != null) {
            this.threads = getCommunityDao().findThreads(getId());
        }
        return threads;
    }

    @Override
    public List<Thread> getThreadsOrderByLastUpdateDateDesc() {
        List<Thread> l = getThreads();
        if (l == null) {
            return l;
        } else {
            Collections.sort(l, new Comparator<Thread>() {
                @Override
                public int compare(Thread t1, Thread t2) {
                    return t2.getLastUpdateDate().compareTo(
                            t1.getLastUpdateDate());
                }
            });
            return l;
        }
    }

    @Override
    public void deleteThread(int threadId)
        throws CannotDeleteThreadException {
        Thread t = findThread(threadId);
        if (t == null) {
            throw new CannotDeleteThreadException("The thread ID '" + threadId
                    + "' not found.");
        }
        t.delete();
    }

    /**
     * 폜\ł邩ǂ𔻒肵܂B̃\bh <code>true</code> ԂꍇAf[^̐
     * Ɉᔽ邱ƂȂ̃R~jeB폜ł܂B
     *
     * @return 폜\ł <code>true</code> AłȂ <code>false</code>
     */
    public abstract boolean isDeletable();

    /**
     * ۂ̃R~jeBύXWbN܂B
     *
     * @param <C> R~jeB
     * @param clazz ύX̃R~jeBNX
     * @return ύX̃R~jeB
     */
    protected abstract <C extends Community> C doChangeTo(Class<C> clazz);

    @Override
    public Integer getId() {
        return id;
    }

    @Override
    public void setId(Integer id) {
        this.id = id;
    }

    @Override
    public String getCode() {
        return code;
    }

    @Override
    public void setCode(String code) {
        this.code = code;
    }

    @Override
    public Message getOverview() {
        return overview;
    }

    @Override
    public void setOverview(Message overview) {
        this.overview = overview;
    }

    @Override
    public AccessControl getPostAccessControl() {
        return postAccessControl;
    }

    @Override
    public void setPostAccessControl(AccessControl postAccessControl) {
        this.postAccessControl = postAccessControl;
    }

    protected CommunityDao getCommunityDao() {
        return communityDao;
    }

    public void setCommunityDao(CommunityDao communityDao) {
        this.communityDao = communityDao;
    }

    protected DomainFactory getDomainFactory() {
        return domainFactory;
    }

    public void setDomainFactory(DomainFactory domainFactory) {
        this.domainFactory = domainFactory;
    }
}
